@@ -1,9 +1,9 @@
 local base = require("classes.base")
 
-local plain = pl.class(base)
-plain._name = "plain"
+local class = pl.class(base)
+class._name = "plain"
 
-plain.defaultFrameset = {
+class.defaultFrameset = {
   content = {
     left = "5%pw",
     right = "95%pw",
@@ -23,7 +23,7 @@ plain.defaultFrameset = {
     bottom = "90%ph"
   }
 }
-plain.firstContentFrame = "content"
+class.firstContentFrame = "content"
 
 local skips = {
   small = "3pt plus 1pt minus 1pt",
@@ -31,19 +31,18 @@ local skips = {
   big = "12pt plus 4pt minus 4pt"
 }
 
-function plain:_init (options)
-  if self._legacy and not self._deprecated then return self:_deprecator(plain) end
+function class:_init (options)
   base._init(self, options)
   self:loadPackage("bidi")
   self:loadPackage("folio")
-  return self
 end
 
-function plain:declareOptions ()
+function class:declareOptions ()
   base.declareOptions(self)
   self:declareOption("direction", function (_, value)
     if value then
       SILE.documentState.direction = value
+      SILE.settings:set("font.direction", value, true)
       for _, frame in pairs(self.defaultFrameset) do
         if not frame.direction then
           frame.direction = value
@@ -54,12 +53,12 @@ function plain:declareOptions ()
   end)
 end
 
-function plain:setOptions (options)
+function class:setOptions (options)
   -- TODO: set a default direction here?
   base.setOptions(self, options)
 end
 
-function plain:declareSettings ()
+function class:declareSettings ()
   base.declareSettings(self)
   for k, v in pairs(skips) do
     SILE.settings:declare({
@@ -71,11 +70,11 @@ function plain:declareSettings ()
   end
 end
 
-function plain:registerCommands ()
+function class:registerCommands ()
 
   SILE.classes.base.registerCommands(self)
 
-  SILE.registerCommand("noindent", function (_, content)
+  self:registerCommand("noindent", function (_, content)
     if #SILE.typesetter.state.nodes ~= 0 then
       SU.warn("\\noindent called after nodes already recieved in a paragraph, the setting will have no effect because the parindent (if any) has already been output")
     end
@@ -83,162 +82,199 @@ function plain:registerCommands ()
     SILE.process(content)
   end, "Do not add an indent to the start of this paragraph")
 
-  SILE.registerCommand("neverindent", function (_, content)
+  self:registerCommand("neverindent", function (_, content)
     SILE.settings:set("current.parindent", SILE.nodefactory.glue())
     SILE.settings:set("document.parindent", SILE.nodefactory.glue())
     SILE.process(content)
   end, "Turn off all indentation")
 
-  SILE.registerCommand("indent", function (_, content)
+  self:registerCommand("indent", function (_, content)
     SILE.settings:set("current.parindent", SILE.settings:get("document.parindent"))
     SILE.process(content)
   end, "Do add an indent to the start of this paragraph, even if previously told otherwise")
 
   for k, _ in pairs(skips) do
-    SILE.registerCommand(k .. "skip", function (_, _)
+    self:registerCommand(k .. "skip", function (_, _)
       SILE.typesetter:leaveHmode()
       SILE.typesetter:pushExplicitVglue(SILE.settings:get("plain." .. k .. "skipamount"))
     end, "Skip vertically by a " .. k .. " amount")
   end
 
-  SILE.registerCommand("hfill", function (_, _)
+  self:registerCommand("hfill", function (_, _)
     SILE.typesetter:pushExplicitGlue(SILE.nodefactory.hfillglue())
   end, "Add a huge horizontal glue")
 
-  SILE.registerCommand("vfill", function (_, _)
+  self:registerCommand("vfill", function (_, _)
     SILE.typesetter:leaveHmode()
     SILE.typesetter:pushExplicitVglue(SILE.nodefactory.vfillglue())
   end, "Add huge vertical glue")
 
-  SILE.registerCommand("hss", function (_, _)
+  self:registerCommand("hss", function (_, _)
     SILE.typesetter:initline()
     SILE.typesetter:pushGlue(SILE.nodefactory.hssglue())
     table.insert(SILE.typesetter.state.nodes, SILE.nodefactory.zerohbox())
   end, "Add glue which stretches and shrinks horizontally (good for centering)")
 
-  SILE.registerCommand("vss", function (_, _)
+  self:registerCommand("vss", function (_, _)
     SILE.typesetter:pushExplicitVglue(SILE.nodefactory.vssglue())
   end, "Add glue which stretches and shrinks vertically")
 
   local _thinspacewidth = SILE.measurement(0.16667, "em")
 
-  SILE.registerCommand("thinspace", function (_, _)
+  self:registerCommand("thinspace", function (_, _)
     SILE.call("glue", { width = _thinspacewidth })
   end)
 
-  SILE.registerCommand("negthinspace", function (_, _)
+  self:registerCommand("negthinspace", function (_, _)
     SILE.call("glue", { width = -_thinspacewidth })
   end)
 
-  SILE.registerCommand("enspace", function (_, _)
+  self:registerCommand("enspace", function (_, _)
     SILE.call("glue", { width = SILE.measurement(1, "en") })
   end)
 
-  SILE.registerCommand("relax", function (_, _)
+  self:registerCommand("relax", function (_, _)
   end)
 
-  SILE.registerCommand("enskip", function (_, _)
+  self:registerCommand("enskip", function (_, _)
     SILE.call("enspace")
   end)
 
   local _quadwidth = SILE.measurement(1, "em")
 
-  SILE.registerCommand("quad", function (_, _)
+  self:registerCommand("quad", function (_, _)
     SILE.call("glue", { width = _quadwidth })
   end)
 
-  SILE.registerCommand("qquad", function (_, _)
+  self:registerCommand("qquad", function (_, _)
     SILE.call("glue", { width = _quadwidth * 2 })
   end)
 
-  SILE.registerCommand("slash", function (_, _)
+  self:registerCommand("slash", function (_, _)
     SILE.typesetter:typeset("/")
     SILE.call("penalty", { penalty = 50 })
   end)
 
-  SILE.registerCommand("break", function (_, _)
+  self:registerCommand("break", function (_, _)
     SILE.call("penalty", { penalty = -10000 })
-  end)
+  end, "Requests a frame break (if in vertical mode) or a line break (if in horizontal mode)")
 
-  SILE.registerCommand("cr", function (_, _)
+  self:registerCommand("cr", function (_, _)
     SILE.call("hfill")
     SILE.call("break")
-  end)
+  end, "Fills a line with a stretchable glue and then requests a line break")
 
-  SILE.registerCommand("framebreak", function (_, _)
-    SILE.call("break")
-  end)
+  -- Despite their name, in older versions, \framebreak and \pagebreak worked badly in horizontal
+  -- mode. The former was a linebreak, and the latter did nothing. That was surely not intended.
+  -- There are many ways, though to assume what's wrong or what the user's intent ought to be.
+  -- We now warn, and terminate the paragraph, but to all extents this might be a wrong approach to
+  -- reconsider at some point.
 
-  SILE.registerCommand("pagebreak", function (_, _)
-    SILE.call("penalty", { penalty = -20000 })
-  end)
+  self:registerCommand("framebreak", function (_, _)
+    if not SILE.typesetter:vmode() then
+      SU.warn("framebreak was not intended to work in horizontal mode. Behaviour may change in future versions")
+    end
+    SILE.call("penalty", { penalty = -10000, vertical = true })
+  end, "Requests a frame break (switching to vertical mode if needed)")
 
-  SILE.registerCommand("nobreak", function (_, _)
+  self:registerCommand("pagebreak", function (_, _)
+    if not SILE.typesetter:vmode() then
+      SU.warn("pagebreak was not intended to work in horizontal mode. Behaviour may change in future versions")
+    end
+    SILE.call("penalty", { penalty = -20000, vertical = true })
+  end, "Requests a non-negotiable page break (switching to vertical mode if needed)")
+
+  self:registerCommand("nobreak", function (_, _)
     SILE.call("penalty", { penalty = 10000 })
-  end)
+  end, "Inhibits a frame break (if in vertical mode) or a line break (if in horizontal mode)")
 
-  SILE.registerCommand("novbreak", function (_, _)
+  self:registerCommand("novbreak", function (_, _)
     SILE.call("penalty", { penalty = 10000, vertical = true })
-  end)
+  end, "Inhibits a frame break (switching to vertical mode if needed)")
 
-  SILE.registerCommand("allowbreak", function (_, _)
+  self:registerCommand("allowbreak", function (_, _)
     SILE.call("penalty", { penalty = 0 })
-  end)
+  end, "Allows a page break (if in vertical mode) or a line break (if in horizontal mode) at a point would not be considered as suitable for breaking")
 
-  SILE.registerCommand("filbreak", function (_, _)
+  -- THIS SEEMS BROKEN BUT THE COMMAND NOT MENTIONED IN THE SILE MANUAL
+  -- In TeX, "\filbreak" compensates the vertical fill if no break actually occurs
+  -- (\def\filbreak{\par\vfil\penalty-200\vfilneg)
+  self:registerCommand("filbreak", function (_, _)
     SILE.call("vfill")
     SILE.call("penalty", { penalty = -200 })
-  end)
+  end, "I HAVE THE SAME NAME AS A TEX COMMAND BUT DON'T SEEM TO BE THE SAME")
 
-  SILE.registerCommand("goodbreak", function (_, _)
+  -- NOTE: TeX's "\goodbreak" does a \par first, so always switches to vertical mode.
+  -- SILE differs here, allowing it both within a paragraph (line breaking) and between
+  -- paragraphs (page breaking).
+  self:registerCommand("goodbreak", function (_, _)
     SILE.call("penalty", { penalty = -500 })
-  end)
+  end, "Indicates a good potential point to break a frame (if in vertical mode) or a line (if in horizontal mode")
 
-  SILE.registerCommand("eject", function (_, _)
+  self:registerCommand("eject", function (_, _)
     SILE.call("vfill")
     SILE.call("break")
-  end)
+  end, "Fills the page with stretchable vglue and then request a page break")
 
-  SILE.registerCommand("supereject", function (_, _)
+  self:registerCommand("supereject", function (_, _)
     SILE.call("vfill")
     SILE.call("penalty", { penalty = -20000 })
-  end)
+  end, "Fills the page with stretchable vglue and then requests a non-negotiable page break")
 
-  SILE.registerCommand("justified", function (_, content)
+  self:registerCommand("justified", function (_, content)
     SILE.settings:set("document.rskip", nil)
     SILE.settings:set("document.spaceskip", nil)
     SILE.process(content)
     SILE.call("par")
   end)
 
-  SILE.registerCommand("rightalign", function (_, content)
+  self:registerCommand("rightalign", function (_, content)
     SILE.call("raggedleft", {}, function ()
       SILE.process(content)
       SILE.call("par")
     end)
   end)
 
-  SILE.registerCommand("em", function (_, content)
+  self:registerCommand("em", function (_, content)
     SILE.call("font", { style = "Italic" }, content)
   end)
 
-  SILE.registerCommand("strong", function (_, content)
+  self:registerCommand("strong", function (_, content)
     SILE.call("font", { weight = 700 }, content)
   end)
 
-  SILE.registerCommand("nohyphenation", function (_, content)
+  self:registerCommand("code", function (options, content)
+    -- IMPLEMENTATION NOTE:
+    -- The \code command came from the url package, though used in plenty of
+    -- places. It was referring to the verbatim:font from the verbatim
+    -- package, which _should_ be sort of unrelated.
+    -- Trying to untangle the things here, by introducing the
+    -- definition from the former, but it's of sub-quality...
+    -- The ugly -3 size is a HACK of sorts.
+    options.family = options.family or "Hack"
+    options.size = options.size or SILE.settings:get("font.size") - 3
+    SILE.call("font", options, content)
+  end)
+
+  self:registerCommand("nohyphenation", function (_, content)
     SILE.call("font", { language = "und" }, content)
   end)
 
-  SILE.registerCommand("raggedright", function (_, content)
+  self:registerCommand("raggedright", function (_, content)
     SILE.call("ragged", { right = true }, content)
   end)
 
-  SILE.registerCommand("raggedleft", function (_, content)
+  self:registerCommand("raggedleft", function (_, content)
     SILE.call("ragged", { left = true }, content)
   end)
 
-  SILE.registerCommand("quote", function (_, content)
+  self:registerCommand("quote", function (_, content)
+    SU.deprecated("\\quote", "\\pullquote", "0.14.5", "0.16.0", [[
+  The \quote command has *such* bad output it is being completely
+  deprecated as unsuitable for general purpose use. The pullquote
+  package (\use[module=packages.pullquote]) provides one alternative,
+  but you can also copy and adapt the original source from the plain
+  class if you need to maintain exact output past SILE v0.16.0.]])
     SILE.call("smallskip")
     SILE.call("par")
     local margin = SILE.measurement(2.5, "em")
@@ -254,22 +290,27 @@ function plain:registerCommands ()
     SILE.call("smallskip")
   end)
 
-  SILE.registerCommand("listitem", function (_, content)
+  self:registerCommand("listitem", function (_, content)
+    SU.deprecated("\\listitem", "\\item", "0.14.6", "0.16.0", [[
+  The new list package (\use[module=packages.lists) has much better
+  typography for lists. If you want to maintain the exact output of listitem
+  past SILE v0.16.0 copy the source of \listitem from the plain class into
+  your project.]])
     SILE.call("medskip")
     SILE.typesetter:typeset("• ")
     SILE.process(content)
     SILE.call("medskip")
   end)
 
-  SILE.registerCommand("sloppy", function (_, _)
+  self:registerCommand("sloppy", function (_, _)
     SILE.settings:set("linebreak.tolerance", 9999)
   end)
 
-  SILE.registerCommand("awful", function (_, _)
+  self:registerCommand("awful", function (_, _)
     SILE.settings:set("linebreak.tolerance", 10000)
   end)
 
-  SILE.registerCommand("center", function (_, content)
+  self:registerCommand("center", function (_, content)
     if #SILE.typesetter.state.nodes ~= 0 then
       SU.warn("\\center environment started after other nodes in a paragraph, may not center as expected")
     end
@@ -280,7 +321,7 @@ function plain:registerCommands ()
     end)
   end)
 
-  SILE.registerCommand("ragged", function (options, content)
+  self:registerCommand("ragged", function (options, content)
     SILE.settings:temporarily(function ()
       if SU.boolean(options.left, false) then SILE.settings:set("document.lskip", SILE.nodefactory.hfillglue()) end
       if SU.boolean(options.right, false) then SILE.settings:set("document.rskip", SILE.nodefactory.hfillglue()) end
@@ -292,64 +333,22 @@ function plain:registerCommands ()
     end)
   end)
 
-  local _rtl_pre_post = function (box, typesetter, line)
-    local advance = function () typesetter.frame:advanceWritingDirection(box:scaledWidth(line)) end
-    if typesetter.frame:writingDirection() == "RTL" then
-      advance()
-      return function () end
-    else
-      return advance
+  self:registerCommand("hbox", function (_, content)
+    local hbox, hlist = SILE.typesetter:makeHbox(content)
+    SILE.typesetter:pushHbox(hbox)
+    if #hlist > 0 then
+      SU.warn("Hbox has migrating content (ignored for now, but likely to break in future versions)")
+      -- Ugly shim:
+      -- One day we ought to do SILE.typesetter:pushHlist(hlist) here, so as to push
+      -- back the migrating contents from within the hbox'ed content.
+      -- However, old Lua code assumed the hbox to be returned, and sometimes removed it
+      -- from the typesetter queue (for measuring, etc.), assuming it was the last
+      -- element in the queue...
     end
-  end
-
-  SILE.registerCommand("hbox", function (_, content)
-    local index = #(SILE.typesetter.state.nodes)+1
-    local recentContribution = {}
-    SILE.process(content)
-    local l = SILE.length()
-    local h, d = SILE.length(), SILE.length()
-    for i = index, #(SILE.typesetter.state.nodes) do
-      local node = SILE.typesetter.state.nodes[i]
-      if node.is_unshaped then
-        local shape = node:shape()
-        for _, attr in ipairs(shape) do
-          recentContribution[#recentContribution+1] = attr
-          h = attr.height > h and attr.height or h
-          d = attr.depth > d and attr.depth or d
-          l = l + attr:lineContribution():absolute()
-        end
-      else
-        recentContribution[#recentContribution+1] = node
-        l = l + node:lineContribution():absolute()
-        h = node.height > h and node.height or h
-        d = node.depth > d and node.depth or d
-      end
-      SILE.typesetter.state.nodes[i] = nil
-    end
-    local hbox = SILE.nodefactory.hbox({
-        height = h,
-        width = l,
-        depth = d,
-        value = recentContribution,
-        outputYourself = function (self_, typesetter, line)
-          local _post = _rtl_pre_post(self_, typesetter, line)
-          local ox = typesetter.frame.state.cursorX
-          local oy = typesetter.frame.state.cursorY
-          SILE.outputter:setCursor(typesetter.frame.state.cursorX, typesetter.frame.state.cursorY)
-          for _, node in ipairs(self_.value) do
-            node:outputYourself(typesetter, line)
-          end
-          typesetter.frame.state.cursorX = ox
-          typesetter.frame.state.cursorY = oy
-          _post()
-          if SU.debugging("hboxes") then SILE.outputter:debugHbox(self_, self_:scaledWidth(line)) end
-        end
-      })
-    table.insert(SILE.typesetter.state.nodes, hbox)
     return hbox
   end, "Compiles all the enclosed horizontal-mode material into a single hbox")
 
-  SILE.registerCommand("vbox", function (options, content)
+  self:registerCommand("vbox", function (options, content)
     local vbox
     SILE.settings:temporarily(function ()
       if options.width then SILE.settings:set("typesetter.breakwidth", SILE.length(options.width)) end
@@ -364,4 +363,4 @@ function plain:registerCommands ()
 
 end
 
-return plain
+return class
