@@ -1,3 +1,8 @@
+local base = require("packages.base")
+
+local package = pl.class(base)
+package._name = "bibtex"
+
 local lpeg = require("lpeg")
 local epnf = require("epnf")
 
@@ -9,6 +14,7 @@ local balanced = lpeg.C{ "{" * lpeg.P(" ")^0 * lpeg.C(((1 - lpeg.S"{}") + lpeg.V
 local doubleq = lpeg.C( lpeg.P '"' * lpeg.C(((1 - lpeg.S '"\r\n\f\\') + (lpeg.P '\\' * 1)) ^ 0) * '"' )
 
 -- luacheck: push ignore
+---@diagnostic disable: undefined-global, unused-local, lowercase-global
 local bibtexparser = epnf.define(function (_ENV)
   local _ = WS^0
   local sep = lpeg.S(",;") * _
@@ -30,6 +36,7 @@ local bibtexparser = epnf.define(function (_ENV)
   entry = Ct( P("@") * Cg(myTag, "type") * _ * P("{") * _ * Cg(myID, "label") * _ * sep * list * P("}") * _ )
 end)
 -- luacheck: pop
+---@diagnostic enable: undefined-global, unused-local, lowercase-global
 
 local parseBibtex = function (fn)
   fn = SILE.resolveFile(fn)
@@ -50,23 +57,30 @@ local parseBibtex = function (fn)
   return entries
 end
 
-local function init (_, _)
-
+function package:_init ()
+  base._init(self)
   SILE.scratch.bibtex = { bib = {} }
-
   Bibliography = require("packages.bibtex.bibliography")
-
 end
 
-local function registerCommands (_)
+function package.declareSettings (_)
+  SILE.settings:declare({
+    parameter = "bibtex.style",
+    type = "string",
+    default = "chicago",
+    help = "BibTeX style"
+  })
+end
 
-  SILE.registerCommand("loadbibliography", function (options, _)
+function package:registerCommands ()
+
+  self:registerCommand("loadbibliography", function (options, _)
     local file = SU.required(options, "file", "loadbibliography")
     SILE.scratch.bibtex.bib = parseBibtex(file) -- Later we'll do multiple bibliogs, but not now
   end)
 
-  SILE.registerCommand("bibstyle", function (_, content)
-    SU.deprecated("\\bibstyle", 'SILE.settings.set("bibtex.style")', "0.13.2", "0.14.0")
+  self:registerCommand("bibstyle", function (_, content)
+    SU.deprecated("\\bibstyle", '\\set[parameter=bibtex.style]', "0.13.2", "0.14.0")
     if type(content) == "table" then
       content = content[1]
     end
@@ -75,8 +89,8 @@ local function registerCommands (_)
     end
   end)
 
-  SILE.registerCommand("cite", function (options, content)
-    if not options.key then options.key = content[1] end
+  self:registerCommand("cite", function (options, content)
+    if not options.key then options.key = SU.contentToString(content) end
     local style = SILE.settings:get("bibtex.style")
     local bibstyle = require("packages.bibtex.styles." .. style)
     local cite = Bibliography.produceCitation(options, SILE.scratch.bibtex.bib, bibstyle)
@@ -84,63 +98,45 @@ local function registerCommands (_)
       SU.warn("Unknown reference in citation "..options.key)
       return
     end
-    SILE.doTexlike(cite)
+    SILE.processString(("<sile>%s</sile>"):format(cite), "xml")
   end)
 
-  SILE.registerCommand("reference", function (options, content)
-    if not options.key then options.key = content[1] end
+  self:registerCommand("reference", function (options, content)
+    if not options.key then options.key = SU.contentToString(content) end
     local style = SILE.settings:get("bibtex.style")
     local bibstyle = require("packages.bibtex.styles." .. style)
     local cite, err = Bibliography.produceReference(options, SILE.scratch.bibtex.bib, bibstyle)
     if cite == Bibliography.Errors.UNKNOWN_REFERENCE then
-      SU.warn("Unknown reference in citation "..options.key)
+      SU.warn("Unknown reference in citation " .. tostring(options.key))
       return
     end
     if cite == Bibliography.Errors.UNKNOWN_TYPE then
       SU.warn("Unknown type @"..err.." in citation for reference "..options.key)
       return
     end
-    SILE.doTexlike(cite)
+    SILE.processString(("<sile>%s</sile>"):format(cite), "xml")
   end)
 
 end
 
-local function declareSettings (_)
-
-  SILE.settings:declare({
-    parameter = "bibtex.style",
-    type = "string",
-    default = "chicago",
-    help = "BibTeX style"
-  })
-
-end
-
-return {
-  init = init,
-  registerCommands = registerCommands,
-  declareSettings = declareSettings,
-  documentation = [[
+package.documentation = [[
 \begin{document}
-BibTeX is a citation management system. It was originally designed
-for TeX but has since been integrated into a variety of situations.
+BibTeX is a citation management system.
+It was originally designed for TeX but has since been integrated into a variety of situations.
 
-This experimental package allows SILE to read and process BibTeX
-\code{.bib} files and output citations and full text references.
+This experimental package allows SILE to read and process BibTeX \code{.bib} files and output citations and full text references.
 (It doesn’t currently produce full bibliography listings.)
 
-To load a BibTeX file, issue the command
-\autodoc:command{\loadbibliography[file=<whatever.bib>]}
+To load a BibTeX file, issue the command \autodoc:command{\loadbibliography[file=<whatever.bib>]}
 
-To produce an inline citation, call \autodoc:command{\cite{<key>}}, which
-will typeset something like “Jones 1982”. If you want to cite a
-particular page number, use \autodoc:command{\cite[page=22]{<key>}}.
+To produce an inline citation, call \autodoc:command{\cite{<key>}}, which will typeset something like “Jones 1982”.
+If you want to cite a particular page number, use \autodoc:command{\cite[page=22]{<key>}}.
 
 To produce a full reference, use \autodoc:command{\reference{<key>}}.
 
-Currently, the only supported bibliography style is Chicago referencing,
-but other styles should be easy to implement if there is interest.
-Check out \code{packages/bibtex/styles/chicago.lua} and adapt as necessary.
+Currently, the only supported bibliography style is Chicago referencing, but other styles should be easy to implement.
+Adapt \code{packages/bibtex/styles/chicago.lua} as necessary.
 \end{document}
 ]]
-}
+
+return package
